/*************************************************************************\
* Copyright (c) 2006 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* This software is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
\*************************************************************************/

/*
 * MCF5282 Watchdog Timer
 * Once written to, must be written to within 5 seconds to prevent reset.
 */
#include <bsp.h>
#include <epicsStdio.h>
#include <epicsExport.h>
#include <cantProceed.h>
#include <asynDriver.h>
#include <asynInt32.h>

/*
 * Driver private storage
 */
typedef struct drvPvt {
    asynInterface common;
    asynInterface int32;
} drvPvt;

/*
 * asynCommon methods
 */
static void
report(void *pvt, FILE *fp, int details)
{
}

static asynStatus
connect(void *pvt, asynUser *pasynUser)
{
    pasynManager->exceptionConnect(pasynUser);
    return asynSuccess;
}

static asynStatus
disconnect(void *pvt, asynUser *pasynUser)
{
    pasynManager->exceptionDisconnect(pasynUser);
    return asynSuccess;
}
static asynCommon commonMethods = { report, connect, disconnect };

/*
 * int32 methods
 */
static asynStatus
int32Write(void *pvt, asynUser *pasynUser, epicsInt32 value)
{
    MCF5282_WTM_WCR = MCF5282_WTM_WCR_EN;
    MCF5282_WTM_WSR = MCF5282_WTM_WSR_SEQ1;
    MCF5282_WTM_WSR = MCF5282_WTM_WSR_SEQ2;
    return asynSuccess;
}
static asynInt32 int32Methods = { int32Write };

/*
 * Register ourself with ASYN
 */
static void
watchDogDriverRegistrar(void)
{
    drvPvt *pdrvPvt;
    const char *portName = "watchDog";
    unsigned long divider;

    pdrvPvt = callocMustSucceed(sizeof(drvPvt), 1, "watchDog");
    if (pasynManager->registerPort(portName, 0, 1, 0, 0) != asynSuccess) {
        printf("registerPort failed\n");
        return;
    }
    pdrvPvt->common.interfaceType = asynCommonType;
    pdrvPvt->common.pinterface  = &commonMethods;
    pdrvPvt->common.drvPvt = pdrvPvt;
    if (pasynManager->registerInterface(portName, &pdrvPvt->common) != asynSuccess) {
        printf("registerInterface failed\n");
        return;
    }
    pdrvPvt->int32.interfaceType = asynInt32Type;
    pdrvPvt->int32.pinterface  = &int32Methods;
    pdrvPvt->int32.drvPvt = pdrvPvt;
    if (pasynManager->registerInterface(portName, &pdrvPvt->int32) != asynSuccess) {
        printf("registerInterface failed\n");
        return;
    }
    divider = 5 * ((bsp_get_CPU_clock_speed() + 8191) / 8192);
    if (divider > 65535) {
        divider = 65535;
        printf("Warning -- timeout < 5 seconds! (~%ld seconds)\n",
                                divider / ((bsp_get_CPU_clock_speed() / 8192)));
    }
    MCF5282_WTM_WMR = divider;
}
epicsExportRegistrar(watchDogDriverRegistrar);
